@@ -26,7 +26,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: defs.h,v 1.73 2007/01/11 23:19:55 ldv Exp $
+ *	$Id: defs.h,v 1.75 2007/07/24 01:57:11 roland Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -453,7 +453,7 @@ extern void printnum P((struct tcb *, long, char *));
 extern void printnum_int P((struct tcb *, long, char *));
 extern void printpath P((struct tcb *, long));
 extern void printpathn P((struct tcb *, long, int));
-extern void printtv_bitness P((struct tcb *, long, enum bitness_t));
+extern void printtv_bitness P((struct tcb *, long, enum bitness_t, int));
 extern void sprinttv P((struct tcb *, long, enum bitness_t, char *));
 #ifdef HAVE_SIGINFO_T
 extern void printsiginfo P((siginfo_t *, int));
@@ -493,6 +493,7 @@ extern int proc_ioctl P((struct tcb *, int, int));
 extern int stream_ioctl P((struct tcb *, int, int));
 #ifdef LINUX
 extern int rtc_ioctl P((struct tcb *, long, long));
+extern int scsi_ioctl P((struct tcb *, long, long));
 #endif
 
 extern void tv_tv P((struct timeval *, int, int));
@@ -518,7 +519,9 @@ extern int proc_open P((struct tcb *tcp, int attaching));
 	umoven((pid), (addr), sizeof *(objp), (char *) (objp))
 
 #define printtv(tcp, addr)	\
-	printtv_bitness((tcp), (addr), BITNESS_CURRENT)
+	printtv_bitness((tcp), (addr), BITNESS_CURRENT, 0)
+#define printtv_special(tcp, addr)	\
+	printtv_bitness((tcp), (addr), BITNESS_CURRENT, 1)
 
 #ifdef __STDC__
 #ifdef __GNUC__
