@@ -27,7 +27,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: desc.c,v 1.27 2007/01/11 23:19:55 ldv Exp $
+ *	$Id: desc.c,v 1.30 2007/08/02 01:32:17 roland Exp $
  */
 
 #include "defs.h"
@@ -476,7 +476,7 @@ decode_select(struct tcb *tcp, long *args, enum bitness_t bitness)
 		}
 		free(fds);
 		tprintf(", ");
-		printtv_bitness(tcp, args[4], bitness);
+		printtv_bitness(tcp, args[4], bitness, 0);
 	}
 	else
 	{
@@ -669,8 +669,8 @@ struct tcb *tcp;
 	return 0;
 }
 
-int
-sys_epoll_wait(tcp)
+static void
+epoll_wait_common(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp))
@@ -708,6 +708,23 @@ struct tcb *tcp;
 		}
 		tprintf(", %ld, %ld", tcp->u_arg[2], tcp->u_arg[3]);
 	}
+}
+
+int
+sys_epoll_wait(tcp)
+struct tcb *tcp;
+{
+	epoll_wait_common(tcp);
+	return 0;
+}
+
+int
+sys_epoll_pwait(tcp)
+struct tcb *tcp;
+{
+	epoll_wait_common(tcp);
+	if (exiting(tcp))
+		print_sigset(tcp, tcp->u_arg[4], 0);
 	return 0;
 }
 
@@ -899,4 +916,13 @@ sys_pselect6(struct tcb *tcp)
 	}
 	return rc;
 }
+
+int
+sys_eventfd(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp))
+		tprintf("%lu", tcp->u_arg[0]);
+	return 0;
+}
 #endif
