@@ -28,7 +28,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: file.c,v 1.81 2007/01/11 23:19:55 ldv Exp $
+ *	$Id: file.c,v 1.84 2007/07/24 01:57:11 roland Exp $
  */
 
 #include "defs.h"
@@ -1184,6 +1184,7 @@ static const struct xlat fstatatflags[] = {
 	{ AT_SYMLINK_NOFOLLOW,	"AT_SYMLINK_NOFOLLOW"	},
 	{ 0,			NULL			},
 };
+#define utimensatflags fstatatflags
 
 int
 sys_newfstatat(struct tcb *tcp)
@@ -1193,7 +1194,7 @@ sys_newfstatat(struct tcb *tcp)
 		printpath(tcp, tcp->u_arg[1]);
 		tprintf(", ");
 	} else {
-#ifdef HAVE_STAT64
+#if defined HAVE_STAT64 && !(defined POWERPC && defined __powerpc64__)
 		printstat64(tcp, tcp->u_arg[2]);
 #else
 		printstat(tcp, tcp->u_arg[2]);
@@ -2087,19 +2088,30 @@ struct tcb *tcp;
     if (entering(tcp)) {
 	printpath(tcp, tcp->u_arg[0]);
 	tprintf(", ");
-	printtv_bitness(tcp, tcp->u_arg[1], BITNESS_32);
+	printtv_bitness(tcp, tcp->u_arg[1], BITNESS_32,  0);
     }
     return 0;
 }
 #endif
 
 static int
-decode_utimes(struct tcb *tcp, int offset)
+decode_utimes(struct tcb *tcp, int offset, int special)
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[offset]);
 		tprintf(", ");
-		printtv(tcp, tcp->u_arg[offset + 1]);
+		if (tcp->u_arg[offset + 1] == 0)
+			tprintf("NULL");
+		else {
+			tprintf("{");
+			printtv_bitness(tcp, tcp->u_arg[offset + 1],
+					BITNESS_CURRENT, special);
+			tprintf(", ");
+			printtv_bitness(tcp, tcp->u_arg[offset + 1]	
+					+ sizeof (struct timeval),
+					BITNESS_CURRENT, special);
+			tprintf("}");
+		}
 	}
 	return 0;
 }
@@ -2107,7 +2119,7 @@ decode_utimes(struct tcb *tcp, int offset)
 int
 sys_utimes(struct tcb *tcp)
 {
-	return decode_utimes(tcp, 0);
+	return decode_utimes(tcp, 0, 0);
 }
 
 #ifdef LINUX
@@ -2116,7 +2128,19 @@ sys_futimesat(struct tcb *tcp)
 {
 	if (entering(tcp))
 		print_dirfd(tcp->u_arg[0]);
-	return decode_utimes(tcp, 1);
+	return decode_utimes(tcp, 1, 0);
+}
+
+int
+sys_utimensat(struct tcb *tcp)
+{
+	if (entering(tcp)) {
+		print_dirfd(tcp->u_arg[0]);
+		decode_utimes(tcp, 1, 1);
+		tprintf(", ");
+		printflags(utimensatflags, tcp->u_arg[3], "AT_???");
+	}
+	return 0;
 }
 #endif
 
@@ -2124,7 +2148,10 @@ int
 sys_utime(tcp)
 struct tcb *tcp;
 {
-	long ut[2];
+	union {
+		long utl[2];
+		int uti[2];
+	} u;
 
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
@@ -2133,13 +2160,22 @@ struct tcb *tcp;
 			tprintf("NULL");
 		else if (!verbose(tcp))
 			tprintf("%#lx", tcp->u_arg[1]);
-		else if (umoven(tcp, tcp->u_arg[1], sizeof ut,
-		    (char *) ut) < 0)
+		else if (umoven(tcp, tcp->u_arg[1],
+				2 * personality_wordsize[current_personality],
+				(char *) &u) < 0)
 			tprintf("[?, ?]");
-		else {
-			tprintf("[%s,", sprinttime(ut[0]));
-			tprintf(" %s]", sprinttime(ut[1]));
+		else if (personality_wordsize[current_personality]
+			 == sizeof u.utl[0]) {
+			tprintf("[%s,", sprinttime(u.utl[0]));
+			tprintf(" %s]", sprinttime(u.utl[1]));
 		}
+		else if (personality_wordsize[current_personality]
+			 == sizeof u.uti[0]) {
+			tprintf("[%s,", sprinttime(u.uti[0]));
+			tprintf(" %s]", sprinttime(u.uti[1]));
+		}
+		else
+			abort();
 	}
 	return 0;
 }
