@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2001 Hewlett-Packard, Matthew Wilcox
  *
- * $Id: syscallent.h,v 1.17 2006/10/13 20:25:13 ldv Exp $
+ * $Id: syscallent.h,v 1.21 2007/08/02 01:44:14 roland Exp $
  *
  */
 
@@ -134,7 +134,7 @@
 	{ 3,	TS,	sys_sigprocmask,	"sigprocmask"		}, /* 126 */
 	{ 2,	0,	sys_create_module,	"create_module"		}, /* 127 */
 	{ 4,	0,	sys_init_module,	"init_module"		}, /* 128 */
-	{ 1,	0,	sys_delete_module,	"delete_module"		}, /* 129 */
+	{ 2,	0,	sys_delete_module,	"delete_module"		}, /* 129 */
 	{ 1,	0,	sys_get_kernel_syms,	"get_kernel_syms"	}, /* 130 */
 	{ 4,	0,	sys_quotactl,		"quotactl"		}, /* 131 */
 	{ 1,	0,	sys_getpgid,		"getpgid"		}, /* 132 */
@@ -300,3 +300,6 @@
 	{ 4,	TD,	printargs,		"sync_file_range"	}, /* 292 */
 	{ 4,	TD,	printargs,		"tee"			}, /* 293 */
 	{ 4,	TD,	printargs,		"vmsplice"		}, /* 294 */
+	{ 6,	0,	sys_move_pages,		"move_pages"		}, /* 295 */
+	{ 3,	0,	sys_getcpu,		"getcpu"		}, /* 296 */
+	{ 5,	TD,	sys_epoll_pwait,	"epoll_pwait"		}, /* 297 */
