@@ -25,7 +25,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: syscallent.h,v 1.24 2006/12/04 13:46:28 ldv Exp $
+ *	$Id: syscallent.h,v 1.31 2007/08/02 01:44:14 roland Exp $
  */
 
 /*
@@ -940,7 +940,7 @@
 	{ 1,	0,	sys_adjtimex,		"adjtimex"	}, /* 1131 */
 	{ 2,	0,	sys_create_module,	"create_module"	}, /* 1132 */
 	{ 4,	0,	sys_init_module,	"init_module"	}, /* 1133 */
-	{ 1,	0,	sys_delete_module,	"delete_module"	}, /* 1134 */
+	{ 2,	0,	sys_delete_module,	"delete_module"	}, /* 1134 */
 	{ 1,	0,	sys_get_kernel_syms,	"get_kernel_syms"}, /* 1135 */
 	{ 5,	0,	sys_query_module,	"query_module"	}, /* 1136 */
 	{ 4,	0,	sys_quotactl,		"quotactl"	}, /* 1137 */
@@ -1082,7 +1082,7 @@
 	{ 5,	0,	printargs,		"keyctl"	}, /* 1273 */
 	{ 3,	0,	printargs,		"ioprio_set"	}, /* 1274 */
 	{ 2,	0,	printargs,		"ioprio_get"	}, /* 1275 */
-	{ 8,	0,	printargs,		"SYS_1276"	}, /* 1276 */
+	{ 6,	0,	sys_move_pages,		"move_pages"	}, /* 1276 */
 	{ 0,	TD,	printargs,		"inotify_init"	}, /* 1277 */
 	{ 3,	TD,	sys_inotify_add_watch,	"inotify_add_watch" }, /* 1278 */
 	{ 2,	TD,	sys_inotify_rm_watch,	"inotify_rm_watch" }, /* 1279 */
@@ -1110,5 +1110,9 @@
 	{ 4,	TD,	printargs,		"tee"		}, /* 1301 */
 	{ 4,	TD,	printargs,		"vmsplice"	}, /* 1302 */
 	{ 8,	0,	printargs,		"SYS_1303"	}, /* 1303 */
-	{ 8,	0,	printargs,		"SYS_1304"	}, /* 1304 */
-	{ 8,	0,	printargs,		"SYS_1305"	}, /* 1305 */
+	{ 3,	0,	sys_getcpu,		"getcpu"	}, /* 1304 */
+	{ 5,	TD,	sys_epoll_pwait,	"epoll_pwait"	}, /* 1305 */
+	{ 8,	0,	printargs,		"SYS_1306"	}, /* 1306 */
+	{ 3,	TD|TS,	sys_signalfd,		"signalfd"	}, /* 1307 */
+	{ 4,	TD,	sys_timerfd,		"timerfd"	}, /* 1308 */
+	{ 1,	TD,	sys_eventfd,		"eventfd"	}, /* 1309 */
