@@ -25,7 +25,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: syscallent.h,v 1.3 2006/08/22 06:06:13 roland Exp $
+ *	$Id: syscallent.h,v 1.5 2007/07/24 01:53:01 roland Exp $
  */
 
 	{ 0,	0,	sys_restart_syscall,	"restart_syscall" }, /* 0 */
@@ -157,7 +157,7 @@
 	{ 3,	TS,	sys_sigprocmask,	"sigprocmask"	}, /* 126 */
 	{ 2,	0,	sys_create_module,	"create_module"	}, /* 127 */
 	{ 3,	0,	sys_init_module,	"init_module"	}, /* 128 */
-	{ 1,	0,	sys_delete_module,	"delete_module"	}, /* 129 */
+	{ 2,	0,	sys_delete_module,	"delete_module"	}, /* 129 */
 	{ 1,	0,	sys_get_kernel_syms,	"get_kernel_syms"}, /* 130 */
 	{ 4,	0,	sys_quotactl,		"quotactl"	}, /* 131 */
 	{ 1,	0,	sys_getpgid,		"getpgid"	}, /* 132 */
@@ -339,7 +339,7 @@
 	{ 5,	0,	printargs,		"SYS_307"	}, /* 307 */
 	{ 5,	0,	printargs,		"SYS_308"	}, /* 308 */
 	{ 5,	0,	printargs,		"SYS_309"	}, /* 309 */
-	{ 5,	0,	printargs,		"SYS_310"	}, /* 310 */
+	{ 6,	0,	sys_move_pages,		"move_pages"	}, /* 310 */
 	{ 5,	0,	printargs,		"SYS_311"	}, /* 311 */
 	{ 5,	0,	printargs,		"SYS_312"	}, /* 312 */
 	{ 5,	0,	printargs,		"SYS_313"	}, /* 313 */
