@@ -25,7 +25,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: syscallent.h,v 1.25 2006/10/13 20:25:14 ldv Exp $
+ *	$Id: syscallent.h,v 1.32 2007/08/02 01:44:14 roland Exp $
  */
 
 	{ 0,	0,	sys_restart_syscall,	"restart_syscall"	}, /* 0 */
@@ -157,7 +157,7 @@
 	{ 3,	TS,	sys_sigprocmask,	"sigprocmask"		}, /* 126 */
 	{ 2,	0,	sys_create_module,	"create_module"		}, /* 127 */
 	{ 3,	0,	sys_init_module,	"init_module"		}, /* 128 */
-	{ 1,	0,	sys_delete_module,	"delete_module"		}, /* 129 */
+	{ 2,	0,	sys_delete_module,	"delete_module"		}, /* 129 */
 	{ 1,	0,	sys_get_kernel_syms,	"get_kernel_syms"	}, /* 130 */
 	{ 4,	0,	sys_quotactl,		"quotactl"		}, /* 131 */
 	{ 1,	0,	sys_getpgid,		"getpgid"		}, /* 132 */
@@ -330,13 +330,13 @@
 	{ 3,	TD|TF,	sys_faccessat,		"faccessat"		}, /* 298 */
 	{ 2,	0,	printargs,		"set_robust_list"	}, /* 299 */
 	{ 3,	0,	printargs,		"get_robust_list"	}, /* 300 */
-	{ 5,	0,	printargs,		"SYS_301"		}, /* 301 */
-	{ 5,	0,	printargs,		"SYS_302"		}, /* 302 */
-	{ 5,	0,	printargs,		"SYS_303"		}, /* 303 */
+	{ 6,	0,	sys_move_pages,		"move_pages"		}, /* 301 */
+	{ 3,	0,	sys_getcpu,		"getcpu"		}, /* 302 */
+	{ 5,	TD,	sys_epoll_pwait,	"epoll_pwait"		}, /* 303 */
 	{ 5,	0,	printargs,		"SYS_304"		}, /* 304 */
-	{ 5,	0,	printargs,		"SYS_305"		}, /* 305 */
-	{ 5,	0,	printargs,		"SYS_306"		}, /* 306 */
-	{ 5,	0,	printargs,		"SYS_307"		}, /* 307 */
+	{ 3,	TD|TS,	sys_signalfd,		"signalfd"		}, /* 305 */
+	{ 4,	TD,	sys_timerfd,		"timerfd"		}, /* 306 */
+	{ 1,	TD,	sys_eventfd,		"eventfd"		}, /* 307 */
 	{ 5,	0,	printargs,		"SYS_308"		}, /* 308 */
 	{ 5,	0,	printargs,		"SYS_309"		}, /* 309 */
 	{ 5,	0,	printargs,		"SYS_310"		}, /* 310 */
