@@ -34,7 +34,7 @@
 	{ 3,	TD,	sys_write,		"write"		}, /* 4 */
 	{ 3,	TD|TF,	sys_open,		"open"		}, /* 5 */
 	{ 1,	TD,	sys_close,		"close"		}, /* 6 */
-	{ -1,	0,	printargs,		"SYS_7"		}, /* 7 */
+	{ 0,	0,	sys_restart_syscall,	"restart_syscall"}, /* 7 */
 	{ 2,	TD|TF,	sys_creat,		"creat"		}, /* 8 */
 	{ 2,	TF,	sys_link,		"link"		}, /* 9 */
 	{ 1,	TF,	sys_unlink,		"unlink"	}, /* 10 */
@@ -156,7 +156,7 @@
 	{ 3,	TS,	sys_sigprocmask,	"sigprocmask"	}, /* 126 */
 	{ 2,	0,	sys_create_module,	"create_module"	}, /* 127 */
 	{ 3,	0,	sys_init_module,	"init_module"	}, /* 128 */
-	{ 1,	0,	sys_delete_module,	"delete_module"	}, /* 129 */
+	{ 2,	0,	sys_delete_module,	"delete_module"	}, /* 129 */
 	{ 1,	0,	sys_get_kernel_syms,	"get_kernel_syms"}, /* 130 */
 	{ 4,	0,	sys_quotactl,		"quotactl"	}, /* 131 */
 	{ 1,	0,	sys_getpgid,		"getpgid"	}, /* 132 */
@@ -337,16 +337,15 @@
 	{ 4,	TD,	printargs,		"sync_file_range" }, /* 307 */
 	{ 4,	TD,	printargs,		"tee"		}, /* 308 */
 	{ 4,	TD,	printargs,		"vmsplice"	}, /* 309 */
-
-	{ 5,	0,	printargs,		"SYS_310"	}, /* 310 */
-	{ 5,	0,	printargs,		"SYS_311"	}, /* 311 */
-	{ 5,	0,	printargs,		"SYS_312"	}, /* 312 */
+	{ 6,	0,	sys_move_pages,		"move_pages"	}, /* 310 */
+	{ 3,	0,	sys_getcpu,		"getcpu"	}, /* 311 */
+	{ 5,	TD,	sys_epoll_pwait,	"epoll_pwait"	}, /* 312 */
 	{ 5,	0,	printargs,		"SYS_313"	}, /* 313 */
 	{ 5,	0,	printargs,		"SYS_314"	}, /* 314 */
 	{ 5,	0,	printargs,		"SYS_315"	}, /* 315 */
-	{ 5,	0,	printargs,		"SYS_316"	}, /* 316 */
-	{ 5,	0,	printargs,		"SYS_317"	}, /* 317 */
-	{ 5,	0,	printargs,		"SYS_318"	}, /* 318 */
+	{ 3,	TD|TS,	sys_signalfd,		"signalfd"	}, /* 316 */
+	{ 4,	TD,	sys_timerfd,		"timerfd"	}, /* 317 */
+	{ 1,	TD,	sys_eventfd,		"eventfd"	}, /* 318 */
 	{ 5,	0,	printargs,		"SYS_319"	}, /* 319 */
 	{ 5,	0,	printargs,		"SYS_320"	}, /* 320 */
 	{ 5,	0,	printargs,		"SYS_321"	}, /* 321 */
