@@ -27,7 +27,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: syscallent.h,v 1.9 2006/08/22 06:17:33 roland Exp $
+ *	$Id: syscallent.h,v 1.16 2007/08/02 01:44:14 roland Exp $
  */
 
 	{ 0,	0,	sys_restart_syscall,	"restart_syscall"}, /* 0 */
@@ -159,7 +159,7 @@
 	{ 3,	TS,	sys_sigprocmask,	"sigprocmask"	}, /* 126 */
 	{ 2,	0,	sys_create_module,	"create_module"	}, /* 127 */
 	{ 3,	0,	sys_init_module,	"init_module"	}, /* 128 */
-	{ 1,	0,	sys_delete_module,	"delete_module"	}, /* 129 */
+	{ 2,	0,	sys_delete_module,	"delete_module"	}, /* 129 */
 	{ 1,	0,	sys_get_kernel_syms,	"get_kernel_syms"}, /* 130 */
 	{ 4,	0,	sys_quotactl,		"quotactl"	}, /* 131 */
 	{ 1,	0,	sys_getpgid,		"getpgid"	}, /* 132 */
@@ -351,13 +351,13 @@
 	{ 5,	0,	printargs,		"SYS_314"	}, /* 314 */
 	{ 5,	0,	printargs,		"SYS_315"	}, /* 315 */
 	{ 5,	0,	printargs,		"SYS_316"	}, /* 316 */
-	{ 5,	0,	printargs,		"SYS_317"	}, /* 317 */
-	{ 5,	0,	printargs,		"SYS_318"	}, /* 318 */
-	{ 5,	0,	printargs,		"SYS_319"	}, /* 319 */
+	{ 6,	0,	sys_move_pages,		"move_pages"	}, /* 317 */
+	{ 3,	0,	sys_getcpu,		"getcpu"	}, /* 318 */
+	{ 5,	TD,	sys_epoll_pwait,	"epoll_pwait"	}, /* 319 */
 	{ 5,	0,	printargs,		"SYS_320"	}, /* 320 */
-	{ 5,	0,	printargs,		"SYS_321"	}, /* 321 */
-	{ 5,	0,	printargs,		"SYS_322"	}, /* 322 */
-	{ 5,	0,	printargs,		"SYS_323"	}, /* 323 */
+	{ 3,	TD|TS,	sys_signalfd,		"signalfd"	}, /* 321 */
+	{ 4,	TD,	sys_timerfd,		"timerfd"	}, /* 322 */
+	{ 1,	TD,	sys_eventfd,		"eventfd"	}, /* 323 */
 	{ 5,	0,	printargs,		"SYS_324"	}, /* 324 */
 	{ 5,	0,	printargs,		"SYS_325"	}, /* 325 */
 	{ 5,	0,	printargs,		"SYS_326"	}, /* 326 */
