@@ -25,7 +25,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: syscallent.h,v 1.6 2006/08/22 06:17:33 roland Exp $
+ *	$Id: syscallent.h,v 1.13 2007/08/02 01:44:14 roland Exp $
  */
 
 	{ 0,	0,	sys_setup,		"setup"		}, /* 0 */
@@ -157,7 +157,7 @@
 	{ 3,	TS,	sys_sigprocmask,	"sigprocmask"	}, /* 126 */
 	{ 2,	0,	sys_create_module,	"create_module"	}, /* 127 */
 	{ 3,	0,	sys_init_module,	"init_module"	}, /* 128 */
-	{ 1,	0,	sys_delete_module,	"delete_module"	}, /* 129 */
+	{ 2,	0,	sys_delete_module,	"delete_module"	}, /* 129 */
 	{ 1,	0,	sys_get_kernel_syms,	"get_kernel_syms"}, /* 130 */
 	{ 4,	0,	sys_quotactl,		"quotactl"	}, /* 131 */
 	{ 1,	0,	sys_getpgid,		"getpgid"	}, /* 132 */
@@ -373,13 +373,13 @@
 	{ 5,	0,	printargs,		"SYS_342"	}, /* 342 */
 	{ 5,	0,	printargs,		"SYS_343"	}, /* 343 */
 	{ 5,	0,	printargs,		"SYS_344"	}, /* 344 */
-	{ 5,	0,	printargs,		"SYS_345"	}, /* 345 */
-	{ 5,	0,	printargs,		"SYS_346"	}, /* 346 */
-	{ 5,	0,	printargs,		"SYS_347"	}, /* 347 */
+	{ 6,	0,	sys_move_pages,		"move_pages"	}, /* 345 */
+	{ 3,	0,	sys_getcpu,		"getcpu"	}, /* 346 */
+	{ 5,	TD,	sys_epoll_pwait,	"epoll_pwait"	}, /* 347 */
 	{ 5,	0,	printargs,		"SYS_348"	}, /* 348 */
-	{ 5,	0,	printargs,		"SYS_349"	}, /* 349 */
-	{ 5,	0,	printargs,		"SYS_350"	}, /* 350 */
-	{ 5,	0,	printargs,		"SYS_351"	}, /* 351 */
+	{ 3,	TD|TS,	sys_signalfd,		"signalfd"	}, /* 349 */
+	{ 4,	TD,	sys_timerfd,		"timerfd"	}, /* 350 */
+	{ 1,	TD,	sys_eventfd,		"eventfd"	}, /* 351 */
 	{ 5,	0,	printargs,		"SYS_352"	}, /* 352 */
 	{ 5,	0,	printargs,		"SYS_353"	}, /* 353 */
 	{ 5,	0,	printargs,		"SYS_354"	}, /* 354 */
