@@ -1,4 +1,4 @@
-	{ 01,	0,	sys_restart_syscall,"restart_syscall" },/* 0 */
+	{ 0,	0,	sys_restart_syscall, "restart_syscall" },/* 0 */
 	{ 1,	TP,	sys_exit,	"exit" },		/* 1 */
 	{ 0,	TP,	sys_fork,	"fork" },		/* 2 */
 	{ 3,	TD,	sys_read,	"read" },		/* 3 */
@@ -220,7 +220,7 @@
 	{ 1,	0,	sys_adjtimex,	"adjtimex" },		/* 219 */
 	{ 3,	TS,	sys_sigprocmask,"sigprocmask" },	/* 220 */
 	{ 2,	0,	sys_create_module,"create_module" },	/* 221 */
-	{ 1,	0,	sys_delete_module,"delete_module" },
+	{ 2,	0,	sys_delete_module,"delete_module" },
 	{ 1,	0,	sys_get_kernel_syms,"get_kernel_syms"},	/* 223 */
 	{ 1,	0,	sys_getpgid,	"getpgid" },		/* 224 */
 	{ 0,	0,	sys_bdflush,	"bdflush" },		/* 225 */
@@ -266,52 +266,52 @@
 	{ 1,	0,	sys_timer_delete,"timer_delete" },	/* 265 */
 	{ 3,	0,	sys_timer_create,"timer_create" },	/* 266 */
 	{ 5,	0,	printargs,	"SYS_267" },		/* 267 */
-	{ 5,	0,	printargs,	"SYS_268" },		/* 268 */
-	{ 5,	0,	printargs,	"SYS_269" },		/* 269 */
-	{ 5,	0,	printargs,	"SYS_270" },		/* 270 */
-	{ 5,	0,	printargs,	"SYS_271" },		/* 271 */
-	{ 5,	0,	printargs,	"SYS_272" },		/* 272 */
-	{ 5,	0,	printargs,	"SYS_273" },		/* 273 */
-	{ 5,	0,	printargs,	"SYS_274" },		/* 274 */
-	{ 5,	0,	printargs,	"SYS_275" },		/* 275 */
-	{ 5,	0,	printargs,	"SYS_276" },		/* 276 */
-	{ 5,	0,	printargs,	"SYS_277" },		/* 277 */
-	{ 5,	0,	printargs,	"SYS_278" },		/* 278 */
-	{ 5,	0,	printargs,	"SYS_279" },		/* 279 */
-	{ 5,	0,	printargs,	"SYS_280" },		/* 280 */
-	{ 5,	0,	printargs,	"SYS_281" },		/* 281 */
-	{ 5,	0,	printargs,	"SYS_282" },		/* 282 */
-	{ 5,	0,	printargs,	"SYS_283" },		/* 283 */
-	{ 5,	0,	printargs,	"SYS_284" },		/* 284 */
-	{ 5,	0,	printargs,	"SYS_285" },		/* 285 */
-	{ 5,	0,	printargs,	"SYS_286" },		/* 286 */
-	{ 5,	0,	printargs,	"SYS_287" },		/* 287 */
-	{ 5,	0,	printargs,	"SYS_288" },		/* 288 */
-	{ 5,	0,	printargs,	"SYS_289" },		/* 289 */
-	{ 5,	0,	printargs,	"SYS_290" },		/* 290 */
-	{ 5,	0,	printargs,	"SYS_291" },		/* 291 */
-	{ 5,	0,	printargs,	"SYS_292" },		/* 292 */
-	{ 5,	0,	printargs,	"SYS_293" },		/* 293 */
-	{ 5,	0,	printargs,	"SYS_294" },		/* 294 */
-	{ 5,	0,	printargs,	"SYS_295" },		/* 295 */
-	{ 5,	0,	printargs,	"SYS_296" },		/* 296 */
-	{ 5,	0,	printargs,	"SYS_297" },		/* 297 */
-	{ 5,	0,	printargs,	"SYS_298" },		/* 298 */
-	{ 5,	0,	printargs,	"SYS_299" },		/* 299 */
-	{ 5,	0,	printargs,	"SYS_300" },		/* 300 */
-	{ 5,	0,	printargs,	"SYS_301" },		/* 301 */
+	{ 2,	0,	sys_io_setup,		"io_setup"	}, /* 268 */
+	{ 1,	0,	sys_io_destroy,		"io_destroy"	}, /* 269 */
+	{ 3,	0,	sys_io_submit,		"io_submit"	}, /* 270 */
+	{ 3,	0,	sys_io_cancel,		"io_cancel"	}, /* 271 */
+	{ 5,	0,	sys_io_getevents,	"io_getevents"	}, /* 272 */
+	{ 4,	0,	sys_mq_open,		"mq_open"	}, /* 273 */
+	{ 1,	0,	sys_mq_unlink,		"mq_unlink"	}, /* 274 */
+	{ 5,	0,	sys_mq_timedsend,	"mq_timedsend"	}, /* 275 */
+	{ 5,	0,	sys_mq_timedreceive,	"mq_timedreceive" }, /* 276 */
+	{ 2,	0,	sys_mq_notify,		"mq_notify"	}, /* 277 */
+	{ 3,	0,	sys_mq_getsetattr,	"mq_getsetattr"	}, /* 278 */
+	{ 5,	TP,	sys_waitid,		"waitid"	}, /* 279 */
+	{ 4,	TD,	printargs,		"tee"		}, /* 280 */
+	{ 5,	0,	printargs,		"add_key"	}, /* 281 */
+	{ 4,	0,	printargs,		"request_key"	}, /* 282 */
+	{ 5,	0,	printargs,		"keyctl"	}, /* 283 */
+	{ 4,	TD|TF,	sys_openat,		"openat"	}, /* 284 */
+	{ 3,	TD|TF,	sys_mkdirat,		"mkdirat"	}, /* 285 */
+	{ 4,	TD|TF,	sys_mknodat,		"mknodat"	}, /* 286 */
+	{ 5,	TD|TF,	sys_fchownat,		"fchownat"	}, /* 287 */
+	{ 3,	TD|TF,	sys_futimesat,		"futimesat"	}, /* 288 */
+	{ 4,	TD|TD,	sys_newfstatat,		"fstatat64"	}, /* 289 */
+	{ 3,	TD|TF,	sys_unlinkat,		"unlinkat"	}, /* 290 */
+	{ 4,	TD|TF,	sys_renameat,		"renameat"	}, /* 291 */
+	{ 5,	TD|TF,	sys_linkat,		"linkat"	}, /* 292 */
+	{ 3,	TD|TF,	sys_symlinkat,		"symlinkat"	}, /* 293 */
+	{ 4,	TD|TF,	sys_readlinkat,		"readlinkat"	}, /* 294 */
+	{ 3,	TD|TF,	sys_fchmodat,		"fchmodat"	}, /* 295 */
+	{ 3,	TD|TF,	sys_faccessat,		"faccessat"	}, /* 296 */
+	{ 6,	TD,	sys_pselect6,		"pselect6"	}, /* 297 */
+	{ 5,	TD,	sys_ppoll,		"ppoll"		}, /* 298 */
+	{ 1,	TP,	sys_unshare,		"unshare"	}, /* 299 */
+	{ 2,	0,	printargs,		"set_robust_list" }, /* 300 */
+	{ 3,	0,	printargs,		"get_robust_list" }, /* 301 */
 	{ 5,	0,	printargs,	"SYS_302" },		/* 302 */
 	{ 5,	0,	printargs,	"SYS_303" },		/* 303 */
 	{ 5,	0,	printargs,	"SYS_304" },		/* 304 */
 	{ 5,	0,	printargs,	"SYS_305" },		/* 305 */
 	{ 5,	0,	printargs,	"SYS_306" },		/* 306 */
-	{ 5,	0,	printargs,	"SYS_307" },		/* 307 */
-	{ 5,	0,	printargs,	"SYS_308" },		/* 308 */
-	{ 5,	0,	printargs,	"SYS_309" },		/* 309 */
+	{ 6,	0,	sys_move_pages,	"move_pages" },		/* 307 */
+	{ 3,	0,	sys_getcpu,	"getcpu" },		/* 308 */
+	{ 5,	TD,	sys_epoll_pwait,"epoll_pwait" },	/* 309 */
 	{ 5,	0,	printargs,	"SYS_310" },		/* 310 */
-	{ 5,	0,	printargs,	"SYS_311" },		/* 311 */
-	{ 5,	0,	printargs,	"SYS_312" },		/* 312 */
-	{ 5,	0,	printargs,	"SYS_313" },		/* 313 */
+	{ 3,	TD|TS,	sys_signalfd,	"signalfd"},		/* 311 */
+	{ 4,	TD,	sys_timerfd,	"timerfd" },		/* 312 */
+	{ 1,	TD,	sys_eventfd,	"eventfd" },		/* 313 */
 	{ 5,	0,	printargs,	"SYS_314" },		/* 314 */
 	{ 5,	0,	printargs,	"SYS_315" },		/* 315 */
 	{ 5,	0,	printargs,	"SYS_316" },		/* 316 */
