@@ -30,7 +30,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: signal.c,v 1.58 2007/01/11 22:05:06 ldv Exp $
+ *	$Id: signal.c,v 1.61 2007/08/02 01:15:59 roland Exp $
  */
 
 #include "defs.h"
@@ -1435,12 +1435,16 @@ struct tcb *tcp;
 		if (umove(tcp, sp, &sc) < 0)
 		  	return 0;
 		tcp->u_arg[0] = 1;
+# ifdef HAVE_STRUCT_SIGCONTEXT_SC_HI2
+		tcp->u_arg[1] = sc.sc_hi2;
+# else
 		tcp->u_arg[1] = sc.sc_sigset;
+# endif
 	} else {
 	  	tcp->u_rval = tcp->u_error = 0;
 		if(tcp->u_arg[0] == 0)
 		  	return 0;
-		tcp->auxstr = sprintsigmask("mask now ", tcp->u_arg[1]);
+		tcp->auxstr = sprintsigmask("mask now ", tcp->u_arg[1], 0);
 		return RVAL_NONE | RVAL_STR;
 	}
 	return 0;
@@ -1968,4 +1972,15 @@ struct tcb *tcp;
 	return 0;
 }
 
+int
+sys_signalfd(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		tprintf("%ld, ", tcp->u_arg[0]);
+		print_sigset(tcp, tcp->u_arg[1], 1);
+		tprintf("%lu", tcp->u_arg[2]);
+	}
+	return 0;
+}
 #endif /* LINUX */
