@@ -26,7 +26,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: time.c,v 1.19 2007/01/13 11:17:38 ldv Exp $
+ *	$Id: time.c,v 1.22 2007/08/02 01:25:34 roland Exp $
  */
 
 #include "defs.h"
@@ -36,6 +36,13 @@
 #include <sys/timex.h>
 #include <linux/ioctl.h>
 #include <linux/rtc.h>
+
+#ifndef UTIME_NOW
+#define UTIME_NOW ((1l << 30) - 1l)
+#endif
+#ifndef UTIME_OMIT
+#define UTIME_OMIT ((1l << 30) - 2l)
+#endif
 #endif /* LINUX */
 
 struct timeval32
@@ -57,7 +64,7 @@ tprint_timeval(struct tcb *tcp, const struct timeval *tv)
 }
 
 void
-printtv_bitness(struct tcb *tcp, long addr, enum bitness_t bitness)
+printtv_bitness(struct tcb *tcp, long addr, enum bitness_t bitness, int special)
 {
 	if (addr == 0)
 		tprintf("NULL");
@@ -75,14 +82,30 @@ printtv_bitness(struct tcb *tcp, long addr, enum bitness_t bitness)
 		{
 			struct timeval32 tv;
 
-			if ((rc = umove(tcp, addr, &tv)) >= 0)
-				tprint_timeval32(tcp, &tv);
+			if ((rc = umove(tcp, addr, &tv)) >= 0) {
+				if (special && tv.tv_sec == 0 &&
+				    tv.tv_usec == UTIME_NOW)
+					tprintf("UTIME_NOW");
+				else if (special && tv.tv_sec == 0 &&
+					 tv.tv_usec == UTIME_OMIT)
+					tprintf("UTIME_OMIT");
+				else
+					tprint_timeval32(tcp, &tv);
+			}
 		} else
 		{
 			struct timeval tv;
 
-			if ((rc = umove(tcp, addr, &tv)) >= 0)
-				tprint_timeval(tcp, &tv);
+			if ((rc = umove(tcp, addr, &tv)) >= 0) {
+				if (special && tv.tv_sec == 0 &&
+				    tv.tv_usec == UTIME_NOW)
+					tprintf("UTIME_NOW");
+				else if (special && tv.tv_sec == 0 &&
+					 tv.tv_usec == UTIME_OMIT)
+					tprintf("UTIME_OMIT");
+				else
+					tprint_timeval(tcp, &tv);
+			}
 		}
 
 		if (rc < 0)
@@ -180,10 +203,10 @@ struct tcb *tcp;
 		    tcp->u_arg[0], tcp->u_arg[1]);
 	    return 0;
 	}
-	printtv_bitness(tcp, tcp->u_arg[0], BITNESS_32);
+	printtv_bitness(tcp, tcp->u_arg[0], BITNESS_32, 0);
 #ifndef SVR4
 	tprintf(", ");
-	printtv_bitness(tcp, tcp->u_arg[1], BITNESS_32);
+	printtv_bitness(tcp, tcp->u_arg[1], BITNESS_32, 0);
 #endif /* !SVR4 */
     }
     return 0;
@@ -210,10 +233,10 @@ sys_osf_settimeofday(tcp)
 struct tcb *tcp;
 {
     if (entering(tcp)) {
-	printtv_bitness(tcp, tcp->u_arg[0], BITNESS_32);
+	printtv_bitness(tcp, tcp->u_arg[0], BITNESS_32, 0);
 #ifndef SVR4
 	tprintf(", ");
-	printtv_bitness(tcp, tcp->u_arg[1], BITNESS_32);
+	printtv_bitness(tcp, tcp->u_arg[1], BITNESS_32, 0);
 #endif /* !SVR4 */
     }
     return 0;
@@ -265,22 +288,24 @@ printitv_bitness(struct tcb *tcp, long addr, enum bitness_t bitness)
 				struct timeval32 it_interval, it_value;
 			} itv;
 
-			if ((rc = umove(tcp, addr, &itv)) >= 0)
+			if ((rc = umove(tcp, addr, &itv)) >= 0) {
 				tprintf("{it_interval=");
 				tprint_timeval32(tcp, &itv.it_interval);
 				tprintf(", it_value=");
 				tprint_timeval32(tcp, &itv.it_value);
 				tprintf("}");
+			}
 		} else
 		{
 			struct itimerval itv;
 
-			if ((rc = umove(tcp, addr, &itv)) >= 0)
+			if ((rc = umove(tcp, addr, &itv)) >= 0)	{
 				tprintf("{it_interval=");
 				tprint_timeval(tcp, &itv.it_interval);
 				tprintf(", it_value=");
 				tprint_timeval(tcp, &itv.it_value);
 				tprintf("}");
+			}
 		}
 
 		if (rc < 0)
@@ -869,4 +894,29 @@ long arg;
 	}
 	return 1;
 }
+
+#ifndef TFD_TIMER_ABSTIME
+#define TFD_TIMER_ABSTIME (1 << 0)
+#endif
+
+static const struct xlat timerfdflags[] = {
+	{ TFD_TIMER_ABSTIME,	"TFD_TIMER_ABSTIME"	},
+	{ 0,			NULL			}
+};
+
+int
+sys_timerfd(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		/* It does not matter that the kernel uses itimerspec.  */
+		tprintf("%ld, ", tcp->u_arg[0]);
+		printxval(clocknames, tcp->u_arg[1], "CLOCK_???");
+		tprintf(", ");
+		printflags(timerfdflags, tcp->u_arg[2], "TFD_???");
+		tprintf(", ");
+		printitv(tcp, tcp->u_arg[3]);
+	}
+	return 0;
+}
 #endif /* LINUX */
