@@ -30,7 +30,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: util.c,v 1.66 2007/01/11 22:05:06 ldv Exp $
+ *	$Id: util.c,v 1.71 2007/08/02 02:22:06 roland Exp $
  */
 
 #include "defs.h"
@@ -193,7 +193,7 @@ struct timeval *tv, *a, *b;
 {
 	tv->tv_sec = a->tv_sec + b->tv_sec;
 	tv->tv_usec = a->tv_usec + b->tv_usec;
-	if (tv->tv_usec > 1000000) {
+	if (tv->tv_usec >= 1000000) {
 		tv->tv_sec++;
 		tv->tv_usec -= 1000000;
 	}
@@ -227,7 +227,7 @@ struct timeval *tv, *a;
 int n;
 {
 	tv->tv_usec = a->tv_usec * n;
-	tv->tv_sec = a->tv_sec * n + a->tv_usec / 1000000;
+	tv->tv_sec = a->tv_sec * n + tv->tv_usec / 1000000;
 	tv->tv_usec %= 1000000;
 }
 
@@ -442,35 +442,39 @@ int len;
 	static char *outstr;
 	int i, n, c, usehex;
 	char *s, *outend;
+	int trunc;
 
 	if (!addr) {
 		tprintf("NULL");
 		return;
 	}
 	if (!str) {
-		if ((str = malloc(max_strlen)) == NULL
-		    || (outstr = malloc(2*max_strlen)) == NULL) {
+		if ((str = malloc(max_strlen + 1)) == NULL
+		    || (outstr = malloc(4*max_strlen
+					+ sizeof "\"\"...")) == NULL) {
 			fprintf(stderr, "out of memory\n");
 			tprintf("%#lx", addr);
 			return;
 		}
 	}
-	outend = outstr + max_strlen * 2 - 10;
+	outend = outstr + max_strlen * 4;
 	if (len < 0) {
-		n = max_strlen;
+		n = max_strlen + 1;
 		if (umovestr(tcp, addr, n, (char *) str) < 0) {
 			tprintf("%#lx", addr);
 			return;
 		}
 	}
 	else {
-		n = MIN(len, max_strlen);
+		n = MIN(len, max_strlen + 1);
 		if (umoven(tcp, addr, n, (char *) str) < 0) {
 			tprintf("%#lx", addr);
 			return;
 		}
 	}
 
+	trunc = n > max_strlen && str[--n] != 0;
+
 	usehex = 0;
 	if (xflag > 1)
 		usehex = 1;
@@ -537,7 +541,7 @@ int len;
 	}
 
 	*s++ = '\"';
-	if (i < len || (len < 0 && (i == n || s > outend))) {
+	if (i < len || (len < 0 && (trunc || s > outend))) {
 		*s++ = '.'; *s++ = '.'; *s++ = '.';
 	}
 	*s = '\0';
@@ -690,7 +694,8 @@ char *laddr;
 				return 0;
 			}
 			/* But if not started, we had a bogus address. */
-			perror("ptrace: umoven");
+			if (addr != 0 && errno != EIO)
+				perror("ptrace: umoven");
 			return -1;
 		}
 		started = 1;
@@ -705,7 +710,7 @@ char *laddr;
 				/* Ran into 'end of memory' - stupid "printpath" */
 				return 0;
 			}
-			if (addr != 0)
+			if (addr != 0 && errno != EIO)
 				perror("ptrace: umoven");
 			return -1;
 		}
@@ -840,7 +845,8 @@ char *laddr;
 				/* Ran into 'end of memory' - stupid "printpath" */
 				return 0;
 			}
-			perror("umovestr");
+			if (addr != 0 && errno != EIO)
+				perror("umovestr");
 			return -1;
 		}
 		started = 1;
@@ -858,7 +864,8 @@ char *laddr;
 				/* Ran into 'end of memory' - stupid "printpath" */
 				return 0;
 			}
-			perror("umovestr");
+			if (addr != 0 && errno != EIO)
+				perror("umovestr");
 			return -1;
 		}
 		started = 1;
